import {
  HttpError,
  HttpStatus,
  r,
  use,
} from '@marblejs/core';
import { throwError } from 'rxjs';
import { catchError, map, mergeMap } from 'rxjs/operators';
import { validateLogin$ } from './auth.validators';
import IUser from '../../interfaces/user';
import { getUserByEmail } from '../user/user.functions';
import { signToken } from './auth.functions';
import { authenticateUser$ } from './auth.middleware';

const login$ = r.pipe(
  r.matchPath('/login'),
  r.matchType('POST'),
  r.useEffect((req$) => req$.pipe(
    use(validateLogin$),
    use(authenticateUser$),
    map((req) => req.body as IUser),
    mergeMap(getUserByEmail),
    map(signToken),
    map((body) => ({ body })),
    catchError((err) => {
      console.log('Error from main effect!', err);
      return throwError(
        new HttpError('Unauthorized', HttpStatus.UNAUTHORIZED),
      );
    }),
  )),
);

export default [
  login$,
];
