import {
  HttpError,
  HttpMiddlewareEffect,
  HttpRequest,
  HttpStatus,
  use,
} from '@marblejs/core';
import { Observable, throwError, of } from 'rxjs';
import { map, mergeMap } from 'rxjs/operators';
import IUser from '../../interfaces/user';
import { authenticateUser, extractToken } from './auth.functions';
import { validateAuthorize$, validateLogin$ } from './auth.validators';

export const authenticateUser$: HttpMiddlewareEffect = (
  req$: Observable<HttpRequest>,
) => req$.pipe(
  use(validateLogin$),
  map((req) => req.body as IUser),
  map(authenticateUser),
  mergeMap((authenticated) => {
    if (authenticated) {
      return req$;
    }

    return throwError(new HttpError('User name or password is not valid', HttpStatus.UNAUTHORIZED));
  }),
);

export const authorizeUser$: HttpMiddlewareEffect = (
  req$: Observable<HttpRequest>,
) => req$.pipe(
  use(validateAuthorize$),
  map((req) => req.headers.authorization),
  map(extractToken),
  mergeMap((authorized) => {
    if (authorized) {
      console.log('authorized');
      return req$;
    }
    
    return throwError(new HttpError('Unauthorized', HttpStatus.UNAUTHORIZED));
  }),
);
