import {
  HttpError,
  HttpStatus,
} from '@marblejs/core';
import {
  mergeMap,
  catchError,
  map,
} from 'rxjs/operators';
import {
  Observable,
  from,
  throwError,
} from 'rxjs';
import bcrypt from 'bcrypt';
import mongoose from 'mongoose';
import IUser from '../../interfaces/user';
import User from '../../models/user';

export const getUsers = (): Observable<IUser[]> => from(User.find());

export const getUser = (id: string): Observable<IUser> => {
  const user$ = from(User.find({ _id: id }).exec())
    .pipe(
      map((users: IUser[]) => {
        const user: IUser = users.length ? users[0] : {} as IUser;

        return user;
      }),
    );

  return user$;
};

export const registerUser = (user: IUser): Observable<IUser> => {
  const { name, email, password } = user;

  const hash$ = from(bcrypt.hash(password, 10)).pipe(
    mergeMap((hash) => {
      const newUser = new User({
        _id: new mongoose.Types.ObjectId(),
        name,
        email,
        password: hash,
        explorationTime: 0,
        problemsSolved: 0,
      });

      const user$ = from(newUser.save()).pipe(
        catchError(() => throwError(
          new HttpError('User could not be created', HttpStatus.INTERNAL_SERVER_ERROR),
        )),
      );

      return user$;
    }),
    catchError(() => throwError(
      new HttpError('User could not be created', HttpStatus.INTERNAL_SERVER_ERROR),
    )),
  );

  return hash$;
};

export const getUserByEmail = (
  { email }: IUser,
): Observable<IUser> => from(User.find({ email }).exec())
  .pipe(
    map((users) => {
      const user = users.length > 0 ? users[0] : {} as IUser;

      return user;
    }),
  );

export const updateUser = (user: IUser) => {
  const {
    email,
    explorationTime,
    problemsSolved,
  } = user;

  return from(
    User.updateOne(
      {
        email,
      },
      {
        explorationTime,
        problemsSolved,
      },
    ),
  ).pipe(
    map((result) => result.ok)
  );
};
