import {
  HttpError,
  HttpMiddlewareEffect,
  HttpRequest,
  HttpStatus,
  use,
} from '@marblejs/core';
import {
  from,
  Observable,
  of,
  throwError,
} from 'rxjs';
import { mergeMap } from 'rxjs/operators';
import User from '../../models/user';
import { validateRegister } from './user.validators';

const userExists$: HttpMiddlewareEffect = (
  req$: Observable<HttpRequest>,
): Observable<HttpRequest> => req$.pipe(
  use(validateRegister),
  mergeMap(
    (req) => {
      const { email } = req.body;
      const exists$ = from(
        User.find({ email }).exec(),
      ).pipe(
        mergeMap((users) => {
          if (users.length > 0) {
            return throwError(new HttpError('User already exists', HttpStatus.BAD_REQUEST));
          }

          return of(req);
        }),
      );

      return exists$;
    },
  ),
);

export default userExists$;
