import {
  Component,
  EventEmitter,
  Input,
  OnInit,
  Output,
} from '@angular/core';
import Transformation from 'src/app/models/matrices/transformation';
import { Cell } from './matrix.types';

@Component({
  selector: 'app-matrix',
  templateUrl: './matrix.component.html',
  styleUrls: ['./matrix.component.scss'],
})
export class MatrixComponent implements OnInit {
  @Input() transformation!: Transformation;

  @Input() isCreation = false;

  @Output() cellClicked = new EventEmitter<void>();

  selectedCell: Cell;

  viewMatrix: string[][];

  ngOnInit() {
    this.viewMatrix = this.transformation.viewMatrix;
  }

  set view(matrix: string[][]) {
    this.viewMatrix = matrix;
  }

  clickedCell(row: number, col: number): void {
    if (!this.isCreation) {
      return;
    }

    if (row > 2) {
      return;
    }

    this.selectedCell = { row, col };

    this.cellClicked.emit();
  }

  getColor(row: number, col: number): string {
    if (row > 2) {
      return '';
    }

    if (this.isCreation) {
      const c = this.selectedCell;

      return c?.row === row && c?.col === col ? 'primary' : '';
    }

    return this.transformation.isValue(row, col) ? 'primary' : '';
  }
}
