import {
  Component,
  EventEmitter,
  Input,
  Output,
} from '@angular/core';
import { Router } from '@angular/router';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { AuthenticationService } from 'src/app/services/authentication.service';
import Constants from 'src/constants';

@Component({
  selector: 'app-page-header',
  templateUrl: './page-header.component.html',
  styleUrls: ['./page-header.component.scss'],
})
export class PageHeaderComponent {
  @Input() title = Constants.appTitle;

  @Input() showNav = false;

  @Input() hasSidebar = false;

  @Output() toggleNav = new EventEmitter<void>();

  constructor(
    private router: Router,
    private auth: AuthenticationService,
  ) { }

  isLoggedIn$: Observable<boolean> = this.auth.currentUser$
    .pipe(
      map((user) => user !== null ? true : false),
    );

  routeToLogin(): void {
    this.router.navigate(['/landing-page/login'], {});
  }

  logout(): void {
    this.auth.logout();
  }
}
