import {
  Component,
  EventEmitter,
  Input,
  Output,
} from '@angular/core';
import { MatSliderChange } from '@angular/material/slider';
import { SliderData, SliderEvent } from 'src/app/models/matrices/transformation.types';

@Component({
  selector: 'app-transformation-controls',
  templateUrl: './transformation-controls.component.html',
  styleUrls: ['./transformation-controls.component.scss'],
})
export class TransformationControlsComponent {
  @Output() sliderChange = new EventEmitter<SliderEvent>();

  @Output() sliderReset = new EventEmitter<SliderEvent>();

  @Input() dimension!: 3 | 2;

  values: SliderData = {};

  max: number;

  min: number;

  step: number;

  getStep(): number {
    return !this.isRotation() ? 0.001 : (4 * Math.PI) / 10000;
  }

  getMax(): number {
    return !this.isRotation() ? 6 : 2 * Math.PI;
  }

  getMin(): number {
    return !this.isRotation() ? -6 : -2 * Math.PI;
  }

  isRotation() {
    return this.includesArray(Object.keys(this.values), ['theta']);
  }

  includesArray<T>(arr1: T[], arr2: T[]): boolean {
    return arr2.some((item: T) => arr1.includes(item));
  }

  setSliders(values: SliderData) {
    this.values = values;

    console.log('values: ', values);

    this.max = this.getMax();
    this.min = this.getMin();
    this.step = this.getStep();
  }

  resetSlider(name: string): void {
    this.values = {};

    this.sliderReset.emit({
      value: 0,
      name,
    });
  }

  updateSlider($event: MatSliderChange, name: string): void {
    const { value } = $event;

    this.sliderChange.emit({
      value,
      name,
    });
  }
}
