import { Component, Inject } from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { TransformationDialogData, TransformationTypes } from 'src/app/models/matrices/transformation.types';

@Component({
  selector: 'app-transformation-dialog',
  templateUrl: './transformation-dialog.component.html',
  styleUrls: ['./transformation-dialog.component.scss'],
})
export class TransformationDialogComponent {
  types = [
    'Rotation',
    'Translation',
    'Scale',
    'Shear',
  ];

  axes = [
    'x',
    'y',
    'z',
  ];

  position = 'before';

  constructor(
    public dialogRef: MatDialogRef<TransformationDialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data: TransformationDialogData,
  ) {
    if (this.data.dimension === 2) {
      this.axes = [
        'x',
        'y',
      ];
    }
  }

  closeAndSave(): void {
    this.data.before = this.position === 'before';

    this.dialogRef.close(this.data);
  }

  showAxis(): boolean {
    const is3d = this.data.dimension === 3;
    const type = TransformationTypes[this.data.type];

    return is3d
      && (type === TransformationTypes.Rotation
      || type === TransformationTypes.Shear);
  }
}
