import Scene from 'src/viewer/scene/scene';
import Shape from 'src/viewer/shapes/shape';
import GlBuffer from 'src/viewer/gl/buffer';
import Mesh from 'src/viewer/gl/mesh';

const coordinateMesh = (): Mesh => {
  const xColor: [number, number, number, number] = [0.6, 0.0, 0.0, 1];
  const yColor: [number, number, number, number] = [0.0, 0.6, 0.0, 1];
  const zColor: [number, number, number, number] = [0.0, 0.0, 0.6, 1];

  const axisMesh = Shape.createLines({
    xMain: {
      line: [0, 0, 0, 3, 0, 0],
      color: xColor,
    },
    xArrowD: {
      line: [3, 0, 0, 2.95, -0.05, 0],
      color: xColor,
    },
    xArrowU: {
      line: [3, 0, 0, 2.95, 0.05, 0],
      color: xColor,
    },
    yMain: {
      line: [0, 0, 0, 0, 3, 0],
      color: yColor,
    },
    yArrowD: {
      line: [0, 3, 0, -0.05, 2.95, 0],
      color: yColor,
    },
    yArrowU: {
      line: [0, 3, 0, 0.05, 2.95, 0],
      color: yColor,
    },
    zMain: {
      line: [0, 0, 0, 0, 0, 3],
      color: zColor,
    },
    zArrowD: {
      line: [0, 0, 3, 0, -0.05, 2.95],
      color: zColor,
    },
    zArrowU: {
      line: [0, 0, 3, 0, 0.05, 2.95],
      color: zColor,
    },
  });

  return axisMesh;
};

export const createScene = (gl: WebGL2RenderingContext, scene: Scene) => {
  const cubeMesh = Shape.createCube();
  const cubeBuffer = new GlBuffer(gl, cubeMesh, false);

  /* const targetCubeMesh = Shape.createCube();
  const targetCubeBuffer = new GlBuffer(gl, targetCubeMesh, false); */

  const axisMesh = coordinateMesh();
  const axisBuffer = new GlBuffer(gl, axisMesh, false);
  axisMesh.drawMode = gl.LINES;

  scene.addNode(
    'main',
    cubeMesh,
    [0.0, 0.0, 0.0],
    [0.0, 0.0, 0.0],
    [1.0, 1.0, 1.0],
    [0.0, 0.0, 0.0],
    cubeBuffer,
    scene.programs.default,
    'cube',
  );

  /* scene.addNode(
    'target',
    targetCubeMesh,
    [0.0, 0.0, 0.0],
    [0.0, 0.0, 0.0],
    [1.0, 1.0, 3.0],
    [0.0, 0.0, 0.0],
    targetCubeBuffer,
    scene.programs.default,
  ); */

  scene.addNode(
    'axis',
    axisMesh,
    [0.0, 0.0, 0.0],
    [0.0, 0.0, 0.0],
    [1.0, 1.0, 1.0],
    [0.0, 0.0, 0.0],
    axisBuffer,
    scene.programs.line,
  );
};

export const getSphericalCoordinates = (theta: number, phi: number, rho: number): number[] => {
  const x = rho * Math.sin(phi) * Math.cos(theta);
  const y = rho * Math.sin(phi) * Math.sin(theta);
  const z = rho * Math.cos(phi);

  return [x, y, z];
};
