import { Component, OnDestroy } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { Subscription } from 'rxjs';
import { AuthenticationService } from 'src/app/services/authentication.service';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.scss'],
})
export class LoginComponent implements OnDestroy {
  form: FormGroup = new FormGroup({
    email: new FormControl('', [Validators.required, Validators.pattern(/[a-zA-Z.]+@[a-zA-Z][a-zA-Z.]*\.[a-zA-Z]+/)]),
    password: new FormControl('', Validators.required),
  });

  subs = new Subscription();

  get fg() {
    return this.form.controls;
  }

  constructor(
    private auth: AuthenticationService,
    private router: Router,
  ) { }

  ngOnDestroy(): void {
    this.subs.unsubscribe();
  }

  onSubmit(): void {
    this.subs.add(this.auth.login(this.form.value)
      .subscribe(() => {
        this.router.navigate(['activities']);
      }));
  }
}
