import {
  Component, ElementRef, OnDestroy, OnInit, ViewChild,
} from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { Subscription } from 'rxjs';
import { UserService } from 'src/app/services/user.service';
import { mustMatch, haveSpecial, haveCaps } from 'src/app/utils/form-validators';

@Component({
  selector: 'app-register',
  templateUrl: './register.component.html',
  styleUrls: ['./register.component.scss'],
})
export class RegisterComponent implements OnInit, OnDestroy {
  @ViewChild('password') password?: ElementRef<HTMLInputElement>;

  registered = false;

  form: FormGroup = new FormGroup({
    email: new FormControl('', [
      Validators.required,
      Validators.pattern(/[a-zA-Z]+@[a-zA-Z]+\.[a-zA-Z]+/),
      Validators.minLength(8),
    ]),
    name: new FormControl('', [
      Validators.required,
    ]),
    password: new FormControl('', [
      Validators.minLength(8),
      Validators.maxLength(16),
      Validators.required,
      haveSpecial(1),
      haveCaps(2),
    ]),
    confirmedPassword: new FormControl(''),
  });

  subs = new Subscription();

  constructor(
    private userService: UserService,
  ) { }

  ngOnInit(): void {
    this.form.controls.confirmedPassword.setValidators(
      [Validators.required, mustMatch(this.form.controls.password as FormControl)],
    );
  }

  ngOnDestroy(): void {
    this.subs.unsubscribe();
  }

  get fg() {
    return this.form.controls;
  }

  onSubmit() {
    this.subs.add(
      this.userService.register(this.form.value)
        .subscribe((user) => {
          if (user) {
            this.registered = true;
          }
        }),
    );
  }
}
