export default class Matrix {
  _values: number[];

  _dimension: number;

  get dimension(): number {
    return this._dimension;
  }

  constructor(values: number[]) {
    this._values = values;
    this._dimension = this.calculateDimension();
  }

  get viewMatrix(): number[][] {
    return this._values.reduce((prev, _, i, matrix) => {
      if (i % this.dimension === 0) {
        prev.push(matrix.slice(i, i + this.dimension));
      }

      return prev;
    }, [] as number[][]);
  }

  get columnMajor(): number[] {
    const vals = this._values;

    return [
      vals[0],
      vals[4],
      vals[8],
      vals[12],
      vals[1],
      vals[5],
      vals[9],
      vals[13],
      vals[2],
      vals[6],
      vals[10],
      vals[14],
      vals[3],
      vals[7],
      vals[11],
      vals[15],
    ];
  }

  private calculateDimension(): number {
    return Math.sqrt(this._values.length);
  }

  private index(row: number, col: number): number {
    return row * this._dimension + col;
  }

  getValue(row: number, col: number): number {
    return this._values[this.index(row, col)];
  }

  setValue(value: number, row: number, col: number): void {
    this._values[this.index(row, col)] = value;
  }
}
