import { SliderData } from './transformation.types';
import Transformation3d from './transformation3d';

export default class Rotation3d extends Transformation3d {
  private _theta: number;

  constructor() {
    super();

    this._theta = 0;
  }

  set theta(theta: number) {
    this._theta = theta;

    this.setValue(
      Math.cos(theta),
      0,
      0,
    );

    this.setValue(
      -Math.sin(theta),
      0,
      1,
    );

    this.setValue(
      Math.sin(theta),
      1,
      0,
    );

    this.setValue(
      Math.cos(theta),
      1,
      1,
    );
  }

  get theta() {
    return this._theta;
  }

  get values(): SliderData {
    return {
      theta: this.theta,
    };
  }

  set(name: string, value: number | null): void {
    if (!value) {
      return;
    }

    if (name === 'theta') {
      this.theta = value;
    }
  }

  isValue(row: number, col: number): boolean {
    const is1 = row === 0 && col === 0;
    const is2 = row === 0 && col === 1;
    const is3 = row === 1 && col === 0;
    const is4 = row === 1 && col === 1;

    return is1 || is2 || is3 || is4;
  }

  resetValues() {
    this.theta = 0;
  }
}
