import Transformation3d from './transformation3d';
import { Axis3d, SliderData } from './transformation.types';

export default class Rotation3d extends Transformation3d {
  private _axis: Axis3d;

  private _theta: number;

  get axis() {
    return this._axis;
  }

  constructor(axis: Axis3d) {
    super();

    this._axis = axis;
    this._theta = 0;
  }

  get theta() {
    return this._theta;
  }

  set theta(theta: number) {
    this._theta = theta;

    switch (this._axis) {
      case 'y':
        this.setValue(
          Math.cos(theta),
          0,
          0,
        );
        this.setValue(
          Math.sin(theta),
          0,
          2,
        );
        this.setValue(
          -Math.sin(theta),
          2,
          0,
        );
        this.setValue(
          Math.cos(theta),
          2,
          2,
        );
        break;

      case 'z':
        this.setValue(
          Math.cos(theta),
          0,
          0,
        );
        this.setValue(
          Math.sin(theta),
          1,
          0,
        );
        this.setValue(
          -Math.sin(theta),
          0,
          1,
        );
        this.setValue(
          Math.cos(theta),
          1,
          1,
        );
        break;

      default:
        this.setValue(
          Math.cos(theta),
          1,
          1,
        );
        this.setValue(
          -Math.sin(theta),
          1,
          2,
        );
        this.setValue(
          Math.sin(theta),
          2,
          1,
        );
        this.setValue(
          Math.cos(theta),
          2,
          2,
        );
        break;
    }
  }

  get viewMatrix(): string[][] {
    const matrix = super.viewMatrix;
    const theta = Math.floor(this._theta * 100) / 100;

    switch (this._axis) {
      case 'y':
        matrix[0][0] = `cos(${theta})`;
        matrix[0][2] = `sin(${theta})`;
        matrix[2][0] = `-sin(${theta})`;
        matrix[2][2] = `cos(${theta})`;
        break;

      case 'z':
        matrix[0][0] = `cos(${theta})`;
        matrix[0][1] = `-sin(${theta})`;
        matrix[1][0] = `sin(${theta})`;
        matrix[1][1] = `cos(${theta})`;
        break;

      default:
        matrix[1][1] = `cos(${theta})`;
        matrix[2][1] = `sin(${theta})`;
        matrix[1][2] = `-sin(${theta})`;
        matrix[2][2] = `cos(${theta})`;
        break;
    }

    return matrix;
  }

  get values(): SliderData {
    return {
      theta: this.theta,
    };
  }

  set(name: string, value: number | null): void {
    if (value === null) {
      return;
    }

    if (name === 'theta') {
      this.theta = value;
    }
  }

  isValue(row: number, col: number): boolean {
    let is1 = false;
    let is2 = false;
    let is3 = false;
    let is4 = false;

    switch (this._axis) {
      case 'y':
        is1 = row === 0 && col === 0;
        is2 = row === 0 && col === 2;
        is3 = row === 2 && col === 0;
        is4 = row === 2 && col === 2;
        break;

      case 'z':
        is1 = row === 0 && col === 0;
        is2 = row === 0 && col === 1;
        is3 = row === 1 && col === 0;
        is4 = row === 1 && col === 1;
        break;

      case 'x':
        is1 = row === 1 && col === 1;
        is2 = row === 2 && col === 1;
        is3 = row === 1 && col === 2;
        is4 = row === 2 && col === 2;
        break;

      default:
        break;
    }

    return is1 || is2 || is3 || is4;
  }

  resetValues(): void {
    this.theta = 0;
  }
}
