import Transformation3d from './transformation3d';
import { Axis3d, SliderData } from './transformation.types';

export default class Shear3d extends Transformation3d {
  private _axis: Axis3d;

  constructor(axis: Axis3d) {
    super();

    this._axis = axis;
  }

  get x1() {
    switch (this._axis) {
      case 'y':
        return this.getValue(
          1,
          0,
        );

      case 'z':
        return this.getValue(
          2,
          0,
        );

      default:
        return this.getValue(
          1,
          0,
        );
    }
  }

  get x2() {
    switch (this._axis) {
      case 'y':
        return this.getValue(
          1,
          3,
        );

      case 'z':
        return this.getValue(
          2,
          1,
        );

      default:
        return this.getValue(
          1,
          2,
        );
    }
  }

  set x1(x1: number) {
    switch (this._axis) {
      case 'y':
        this.setValue(
          x1,
          1,
          0,
        );
        break;

      case 'z':
        this.setValue(
          x1,
          2,
          0,
        );
        break;

      default:
        this.setValue(
          x1,
          1,
          0,
        );
        break;
    }
  }

  set x2(x2: number) {
    switch (this._axis) {
      case 'y':
        this.setValue(
          x2,
          1,
          3,
        );
        break;

      case 'z':
        this.setValue(
          x2,
          2,
          1,
        );
        break;

      default:
        this.setValue(
          x2,
          1,
          2,
        );
        break;
    }
  }

  get values(): SliderData {
    return {
      x1: this.x1,
      x2: this.x2,
    };
  }

  set(name: string, value: number | null): void {
    if (value === null) {
      return;
    }

    switch (name) {
      case 'x1':
        this.x1 = value;
        break;

      case 'x2':
        this.x2 = value;
        break;

      default:
        break;
    }
  }

  isValue(row: number, col: number): boolean {
    let isX1 = false;
    let isX2 = false;

    switch (this._axis) {
      case 'y':
        isX1 = row === 1 && col === 0;
        isX2 = row === 1 && col === 3;
        break;

      case 'z':
        isX1 = row === 2 && col === 1;
        isX2 = row === 2 && col === 0;
        break;

      case 'x':
        isX1 = row === 1 && col === 0;
        isX2 = row === 1 && col === 2;
        break;

      default:
        break;
    }

    return isX1 || isX2;
  }

  resetValues() {
    this.x1 = 0;
    this.x2 = 0;
  }
}
