import Matrix from './matrix';
import { SliderData } from './transformation.types';

interface StringIndexed {
  [name: string]: any;
}

export default abstract class Transformation implements StringIndexed {
  protected _matrix: Matrix;

  getValue(row: number, col: number): number {
    return this._matrix.getValue(row, col);
  }

  setValue(value: number, row: number, col: number): void {
    this._matrix.setValue(value, row, col);
  }

  get matrix() {
    return this._matrix;
  }

  get viewMatrix(): string[][] {
    return this.matrix.viewMatrix.map((row: number[]) => row.map((item: number) => String(item)));
  }

  abstract get values(): SliderData;

  abstract set(name: string, value: number | null): void;

  abstract isValue(row: number, col: number): boolean;

  abstract resetValues(): void;
}
