import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { BehaviorSubject, Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import User from 'src/app/models/User';
import HttpService from 'src/app/services/http.service';

@Injectable({
  providedIn: 'root',
})
export class AuthenticationService extends HttpService {
  protected namespace = 'auth';

  currentUserSubject$ = new BehaviorSubject<User | null>(JSON.parse(localStorage.getItem('currentUser') as string));

  currentUser$ = this.currentUserSubject$.asObservable();

  constructor(
    protected http: HttpClient,
    protected router: Router,
  ) {
    super(http);
  }

  get currentUserValue(): User {
    return this.currentUserSubject$.value as User;
  }

  login(user: User): Observable<User> {
    return this.http.post<any>(`${this.apiUrl}/${this.namespace}/login`, user)
      .pipe(
        map((res) => {
          const resUser = res?.user;

          resUser.token = res?.token;

          localStorage.setItem('currentUser', JSON.stringify(resUser));
          this.currentUserSubject$.next(resUser);

          return resUser;
        }),
      );
  }

  logout() {
    localStorage.removeItem('currentUser');
    this.currentUserSubject$.next(null);
    this.router.navigate(['landing-page']);
  }

  isLoggedIn(): boolean {
    return this.currentUserValue !== null;
  }
}
