import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import User from 'src/app/models/User';
import HttpService from 'src/app/services/http.service';
import { AuthenticationService } from './authentication.service';

@Injectable({
  providedIn: 'root',
})
export class UserService extends HttpService {
  protected namespace = 'user';

  constructor(
    protected http: HttpClient,
    protected auth: AuthenticationService,
  ) {
    super(http);
  }

  get(): Observable<User[]> {
    return this.http.get<User[]>(`${this.apiUrl}/${this.namespace}`, this.standardOptions);
  }

  getById(id: string): Observable<User> {
    return this.http.get<User>(`${this.apiUrl}/${this.namespace}/${id}`, this.standardOptions);
  }

  register(user: User): Observable<any> {
    return this.http.post<User>(`${this.apiUrl}/${this.namespace}/register`, user, this.standardOptions);
  }

  update(user: User) {
    return this.http.put<User>(`${this.apiUrl}/${this.namespace}/`, user, this.standardOptions);
  }
}
