import { AbstractControl, FormControl, ValidatorFn } from '@angular/forms';

export const mustMatch = (matching: FormControl): ValidatorFn => (control: AbstractControl) => {
  const mVal = matching.value;
  const cVal = control.value;

  return cVal !== mVal ? {
    hasMismatch: {
      value: false,
    },
  } : null;
};

export const haveSpecial = (nSpecial: number) => {
  const n = Math.floor(nSpecial);
  return (control: AbstractControl) => {
    const { value } = control;
    const hasSpecial = value.split('').reduce((p: number, c: string) => (c.search(/[a-zA-Z]/) === -1 ? p + 1 : p), 0) >= n;
    return !hasSpecial ? {
      notEnoughSpecial: {
        value,
      },
    } : null;
  };
};

export const haveCaps = (nCaps: number) => {
  const n = Math.floor(nCaps);
  return (control: AbstractControl) => {
    const { value } = control;
    const hasSpecial = value.split('').reduce((p: number, c: string) => (c.search(/[A-Z]/) !== -1 ? p + 1 : p), 0) >= n;
    return !hasSpecial ? {
      notEnoughCaps: {
        value,
      },
    } : null;
  };
};
