/**
 *  html element creation
 */
import { Props } from '../types/html';

/**
 * Creates an html element
 * @param {string} type the type of elemen to create
 * @param {Props} attributes attributes to add
 * @param {Props} styles styles to add
 *
 * @returns {HTMLElement} created element
 */
const Element = (
  type: string,
  attributes: Props,
  styles: Props,
): HTMLElement => {
  const element = document.createElement(type);
  Object.keys(attributes)
    .forEach((name) => element.setAttribute(name, attributes[name] as string));
  Object.keys(styles)
    .forEach((name) => element.style.setProperty(name, styles[name] as string));
  return element;
};

export default Element;
