class Mesh {
  private _vertices: number[];

  private _normals: number[];

  private _uvs: number[];

  private _colors: number[];

  private _indices: number[];

  private _drawMode: number;

  /**
   * constructor
   * @param {number[]} vertices vertices of the mesh
   * @param {number[]} normals normals of the mesh
   * @param {number[]} uvs uvs of the mesh
   * @param {number[]} colors  colors of the mesh
   * @param {number[]} indices indices of the mesh
   */
  constructor(
    vertices: number[],
    normals: number[],
    uvs: number[],
    colors: number[],
    indices: number[],
  ) {
    this._vertices = vertices;
    this._normals = normals;
    this._uvs = uvs;
    this._colors = colors;
    this._indices = indices;
  }

  /**
   * gets the vertices for this mesh
   *
   * @returns {number[]} the vertices
   */
  get vertices(): number[] {
    return this._vertices;
  }

  /**
   * gets the normals for this mesh
   *
   * @returns {number[]} the normals
   */
  get normals(): number[] {
    return this._normals;
  }

  get uvs(): number[] {
    return this._uvs;
  }

  /**
   * gets the colors for this mesh
   *
   * @returns {number[]} colors
   */
  get colors(): number[] {
    return this._colors;
  }

  /**
   * gets for indices
   *
   * @returns {number[]} the indices
   */
  get indices(): number[] {
    return this._indices;
  }

  /**
   * gets the vao count for this mesh
   *
   * @returns {number} vaocount
   */
  get vaoCount(): number {
    return this.indices.length;
  }

  set drawMode(mode: number) {
    this._drawMode = mode;
  }

  get drawMode() {
    return this._drawMode;
  }
}

export default Mesh;
