import Program from './gl/program';
import Scene from './scene/scene';

class Renderer {
  private _gl: WebGL2RenderingContext;

  private _programs: {
    [programName: string]: Program;
  } = {};

  private _scene: Scene;

  private _interruption = false;

  private _width: number;

  private _height: number;

  constructor(
    gl: WebGL2RenderingContext,
    programs: Program[],
    scene: Scene,
    width: number,
    height: number,
  ) {
    this._gl = gl;

    programs.forEach((program) => {
      this._programs[program.name] = program;
    });

    this._scene = scene;
    this._width = width;
    this._height = height;

    gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);
    gl.enable(gl.BLEND);
  }

  runRenderLoop(callback?: () => void): void {
    let then = 0;
    let leftOverDelta = 0;

    const renderLoop = (now: DOMHighResTimeStamp) => {
      now *= 0.001;

      this._gl.clearColor(0.9, 0.9, 0.9, 1.0);
      this._gl.clearDepth(1.0);
      this._gl.enable(this._gl.DEPTH_TEST);
      this._gl.depthFunc(this._gl.LEQUAL);

      const delta = now - then + leftOverDelta;

      then = now;

      if (delta > 1 / 15) {
        // this._scene?.updateScene();
        this._scene?.renderScene(false);

        if (callback) {
          callback();
        }

        leftOverDelta = 0;
      } else {
        leftOverDelta = delta;
      }

      requestAnimationFrame(renderLoop);
    };

    requestAnimationFrame(renderLoop);
  }

  interrupt() {
    this._interruption = true;
  }

  get programs() {
    return this._programs;
  }

  addProgram(program: Program) {
    this._programs[program.name] = program;
  }

  get scene() {
    return this._scene;
  }
}

export default Renderer;
