import { mat4, vec3 } from 'gl-matrix';
import SceneNode from './scene-node';

class CameraNode extends SceneNode {
  private _viewMatrix: mat4;

  constructor(
    id: string,
    position: number[],
    rotation: number[],
    scale: number[],
    refPoint: number[],
    children?: SceneNode[],
  ) {
    super(
      id,
      position,
      rotation,
      scale,
      refPoint,
      children,
    );

    this._viewMatrix = mat4.create();
    mat4.identity(this._viewMatrix);
  }

  public calculateViewMatrix() {
    mat4.identity(this._viewMatrix);

    const scaled = vec3.create();

    vec3.copy(scaled, vec3.fromValues(-this.position[0], -this._position[1], -this._position[2]));

    mat4.translate(this._viewMatrix, this._viewMatrix, scaled);

    mat4.rotateX(this._viewMatrix, this._viewMatrix, -this._rotation[0]);
    mat4.rotateY(this._viewMatrix, this._viewMatrix, -this._rotation[1]);
    mat4.rotateZ(this._viewMatrix, this._viewMatrix, -this._rotation[2]);
  }

  rotate(thetas: number[]) {
    const [x, y, z] = this._rotation;
    const [xd, yd, zd] = thetas;

    this._rotation = [
      x + xd,
      y + yd,
      z + zd,
    ];
  }

  translate(distances: number[]) {
    const [x, y, z] = this._position;
    const [xd, yd, zd] = distances;

    this._position = [
      x + xd,
      y + yd,
      z + zd,
    ];
  }

  get viewMatrix() {
    return this._viewMatrix;
  }
}

export default CameraNode;
