import Mesh from '../gl/mesh';
import Lines from '../types/mesh';

class Shape {
  public static createCube(): Mesh {
    const vertices = [
      // front
      -1.0, -1.0, 1.0,
      1.0, -1.0, 1.0,
      1.0, 1.0, 1.0,
      -1.0, 1.0, 1.0,
      // back
      -1.0, -1.0, -1.0,
      -1.0, 1.0, -1.0,
      1.0, 1.0, -1.0,
      1.0, -1.0, -1.0,
      // top
      -1.0, 1.0, -1.0,
      -1.0, 1.0, 1.0,
      1.0, 1.0, 1.0,
      1.0, 1.0, -1.0,
      // bottom
      -1.0, -1.0, -1.0,
      1.0, -1.0, -1.0,
      1.0, -1.0, 1.0,
      -1.0, -1.0, 1.0,
      // right
      1.0, -1.0, -1.0,
      1.0, 1.0, -1.0,
      1.0, 1.0, 1.0,
      1.0, -1.0, 1.0,
      // left
      -1.0, -1.0, -1.0,
      -1.0, -1.0, 1.0,
      -1.0, 1.0, 1.0,
      -1.0, 1.0, -1.0,
    ];

    const uvs = [
      // front
      0.66, 1.0,
      1.0, 1.0,
      1.0, 0.5,
      0.66, 0.5,
      // back
      1.0, 0.5,
      1.0, 0.0,
      0.66, 0.0,
      0.66, 0.5,
      // top
      0.33, 0.5,
      0.33, 1.0,
      0.66, 1.0,
      0.66, 0.5,
      // bottom
      0.33, 0.5,
      0.66, 0.5,
      0.66, 0.0,
      0.33, 0.0,
      // right
      0.33, 0.5,
      0.33, 1.0,
      0.0, 1.0,
      0.0, 0.5,
      // left
      0.0, 0.0,
      0.33, 0.0,
      0.33, 0.5,
      0.0, 0.5,
    ];

    const normals = [
      0.0, 0.0, 1.0,
      0.0, 0.0, 1.0,
      0.0, 0.0, 1.0,
      0.0, 0.0, 1.0,
      0.0, 0.0, -1.0,
      0.0, 0.0, -1.0,
      0.0, 0.0, -1.0,
      0.0, 0.0, -1.0,
      0.0, 1.0, 0.0,
      0.0, 1.0, 0.0,
      0.0, 1.0, 0.0,
      0.0, 1.0, 0.0,
      0.0, -1.0, 0.0,
      0.0, -1.0, 0.0,
      0.0, -1.0, 0.0,
      0.0, -1.0, 0.0,
      1.0, 0.0, 0.0,
      1.0, 0.0, 0.0,
      1.0, 0.0, 0.0,
      1.0, 0.0, 0.0,
      -1.0, 0.0, 0.0,
      -1.0, 0.0, 0.0,
      -1.0, 0.0, 0.0,
      -1.0, 0.0, 0.0,
    ];

    const indices = [
      0, 1, 2, 0, 2, 3, // front
      4, 5, 6, 4, 6, 7, // back
      8, 9, 10, 8, 10, 11, // top
      12, 13, 14, 12, 14, 15, // bottom
      16, 17, 18, 16, 18, 19, // right
      20, 21, 22, 20, 22, 23, // left
    ];

    return new Mesh(
      vertices,
      normals,
      uvs,
      [],
      indices,
    );
  }

  public static createSquare() {
    const vertices = [
      -1.0, -1.0, 0.0,
      1.0, -1.0, 0.0,
      1.0, 1.0, 0.0,
      -1.0, 1.0, 0.0,
    ];

    const uvs = [
      0.0, 1.0,
      1.0, 1.0,
      1.0, 0.0,
      0.0, 0.0,
    ];

    const indices = [
      0, 1, 2,
      0, 2, 3,
    ];

    return new Mesh(
      vertices,
      [],
      uvs,
      [],
      indices,
    );
  }

  public static createLines(lines: Lines): Mesh {
    const vertices = Object.values(lines).reduce((prev, curr) => {
      prev.push(...curr.line);
      return prev;
    }, [] as number[]);

    const colors = Object.values(lines).reduce((prev, curr) => {
      prev.push(...curr.color);
      prev.push(...curr.color);
      return prev;
    }, [] as number[]);

    const indices = Object.values(lines).reduce((prev, _, i) => {
      const [j, k] = [i * 2, i * 2 + 1];

      prev.push(...[j, k]);

      return prev;
    }, [] as number[]);

    return new Mesh(
      vertices,
      [],
      [],
      colors,
      indices,
    );
  }
}

export default Shape;
