/**
 * checks if an object is array
 * @param {any} obj any object
 *
 * @returns {boolean} result of check
 */
export const isArray = (obj: any): boolean => {
  if (typeof Array.isArray === 'undefined') {
    return Object.prototype.toString.call(obj) === '[object Array]';
  }
  return Array.isArray(obj);
};

/**
 * compares two arrays
 * @param {any[]} a1 first array
 * @param {any[]} a2 second array
 *
 * @returns {boolean} result of comparison
 */
export const arrayCompare = (a1: any[], a2: any[]): boolean => {
  // check if they are arrays
  if (!isArray(a1) || !isArray(a2)) return false;
  // check if their the same length
  if (a1.length !== a2.length) return false;
  // check element by element
  for (let i = 0; i < a1.length; i++) {
    if (a1[i] !== a2[i]) return false;
  }
  return true;
};

export const testCompare = (): boolean => {
  const scenarios = [
    {
      input: [
        [1, 1, 1, 1],
        [1, 1, 1, 1],
      ],
      expectedResult: true,
    },
    {
      input: [
        [0, 0, 0, 0],
        [0, 1, 0, 0],
      ],
      expectedResult: false,
    },
    {
      input: [
        [false, 0, 0, 0],
        [0, 0, 0, 0],
      ],
      expectedResult: false,
    },
  ];
  for (let i = 0; i < scenarios.length; i++) {
    const { input, expectedResult } = scenarios[i];
    const result = arrayCompare(input[0], input[1]);
    if (result !== expectedResult) return false;
  }
  return true;
};
